require 'aws-sdk'

module DCell
  module Registry
    class S3Adapter
      def initialize(options)
        # Convert all options to symbols :/
        options = options.inject({}) { |h,(k,v)| h[k.to_sym] = v; h }

        @env = options[:env] || 'production'
        bucket = options[:bucket] || "dcell_#{Celluloid.uuid}_#{@env}"

        access_key_id = options[:access_key_id] || ENV['AWS_ACCESS_KEY_ID']
        secret_access_key = options[:secret_access_key] || ENV['AWS_SECRET_ACCESS_KEY']
        s3_opts = { :access_key_id => access_key_id, :secret_access_key => secret_access_key }
        s3_opts.merge!({ :region => options[:region] }) if options[:region]
        @s3 = AWS::S3.new(s3_opts)

        @bucket = @s3.buckets[bucket]
        @bucket = @s3.buckets.create(bucket) unless @bucket.exists?

        @node_registry   = NodeRegistry.new(@bucket)
        @global_registry = GlobalRegistry.new(@bucket)
      end

      def clear_nodes
        @node_registry.clear
      end

      def clear_globals
        @global_registry.clear
      end

      def delete_store
        @bucket.delete!
      rescue AWS::S3::Errors::NoSuchBucket
        Logger.info "Bucket #{@bucket.name} doesn't exist"
      end

      class NodeRegistry
        def initialize(bucket)
          @bucket = bucket
        end

        def get(node_id)
          @bucket.objects["nodes/#{node_id}"].read
        rescue AWS::S3::Errors::NoSuchKey
          Logger.error "No such node #{node_id}"
          nil
        end

        def set(node_id, addr)
          @bucket.objects["nodes/#{node_id}"].write(addr)
        end

        def nodes
          @bucket.objects.with_prefix('nodes/').inject([]) do |arr, elt|
            elt.key =~ /^nodes\/(.+)$/
            arr << $1
          end
        end

        def clear
          @bucket.objects.with_prefix('nodes/').map(&:delete)
        end
      end

      def get_node(node_id);       @node_registry.get(node_id) end
      def set_node(node_id, addr); @node_registry.set(node_id, addr) end
      def nodes;                   @node_registry.nodes end

      class GlobalRegistry
        def initialize(bucket)
          @bucket = bucket
        end

        def get(key)
          string = @bucket.objects["globals/#{key}"].read
          Marshal.load string if string
        rescue AWS::S3::Errors::NoSuchKey
          Logger.error "No such global key #{key}"
          nil
        end

        # Set a global value
        def set(key, value)
          string = Marshal.dump value
          @bucket.objects["globals/#{key}"].write(string)
        end

        # The keys to all globals in the system
        def global_keys
          @bucket.objects.with_prefix('globals/').inject([]) do |arr, elt|
            elt.key =~ /^globals\/(.+)$/
            arr << $1
          end
        end

        def clear
          @bucket.objects.with_prefix('globals/').map(&:delete)
        end
      end

      def get_global(key);        @global_registry.get(key) end
      def set_global(key, value); @global_registry.set(key, value) end
      def global_keys;            @global_registry.global_keys end
    end
  end
end
